/***************************************************************************
* File Name: main.c
* Processor(s): MSP430F16x (tested), MSP430F14x (not tested)
* Compiler: Code Composer Studio V4 (4.2.5.00005)
* 
* Copyright  2012 Playing With Fusion, Inc.
* SOFTWARE LICENSE AGREEMENT: This code is released under the MIT License. 
* 
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
* DEALINGS IN THE SOFTWARE.
* **************************************************************************
* REVISION HISTORY:
* Author			Date		Comments
* J. Steinlage		2012Dec16	Original version
* 
* **************************************************************************
* ADDITIONAL NOTES:
* This file contains functions to initialize the MSP430 used for the demo
* per the general hardware configuration that can be seen in included
* schematic pwf_an0001_sch1.pdf. Funcionality is as described below:
* 	- Configure MSP430F16x for ~4.9MHz DCO, 38,400 baud UART and ~1.225MHz
* 	  SPI
* 	- Read temperatures from Playing With Fusion MAX31855K-4CH quad channel
* 	  interface board at 250ms intervals
* 	- Broadcast temperature readings as they are obtained in a directly 
* 	  readable format as viewed with a serial interface program 
* 	  (putty, hyperterminal, etc)
* 
***************************************************************************/

#include "msp430x16x.h"
#include "msp430_typeDef.h"
#include "subroutines.h"
#include "msp430_timer_a.h"
#include "hal430_RS232Tx.h"
#include "hal430_spi.h"
#include "hal430_TmpSnsr.h"
#include "pwf_an0001_update_IO.h"
#include "pwf_an0001_defines.h"
#include "pwf_an0001_struct.h"
#include "hal430_clocks_430f1xx.h"
#include "in430.h"
#include "stdio.h"
#include "math.h"

// Define global variables for thermocouples, timer tick
struct var_signed16 temp_TC0 = {999,0,0,333};
struct var_signed16 temp_TC1 = {999,0,0,333};
struct var_signed16 temp_TC2 = {999,0,0,333};
struct var_signed16 temp_TC3 = {999,0,0,333};
unsigned char clk_tick = 0;

// Init Digital Hardware
void init_system(void)
{
	// port setup
	P1OUT = 0x01;			// Turn on 'blinky LED' pin
	P1DIR = 0xFF;				// 0b11111111
	P1SEL = 0x00;				// 0b00000000
								// LED1 output
									// none others populated
	
	P2OUT = CS_TC0 + CS_TC1 + CS_TC2 + CS_TC3;	// Thermocouple chip selects
	P2DIR = 0xFF;				// 0b11111111
	P2SEL = 0x00;				// 0b00000000
								// CS_TC(0/1/2/3) outputs
									// none others populated
	
	P3OUT = 0x00;				// No port function outputs on port 3
	P3DIR = 0x4B;				// 0b01111011    ... 0=input, 1=output (may not matter if peripheral fn set
	P3SEL = 0xCC;				// 0b11001100    ... 1=peripheral fn, 0=port fn
								// UART1_RX input, UART1_TX output, SOMI_USART0, SCLK_USART0
									// none others populated
	
	P5OUT = 0x00;				// no other functions on P5
	P5DIR = 0xFF;				// 0b11111111
	P5SEL = 0x00;				// 0b00000000

	// ports 1 and 2 on F16x has interrupt capability... disable
    P1IE  = 0x00;				// Disable Port 1 interrupts
    P2IE  = 0x00;				// Disable Port 2 interrupts

    P1IFG = 0x00;				// Clear Port 1 interrupt Flags
    P2IFG = 0x00;				// Clear Port 2 interrupt Flags
    
	// Initialize SPI (for thermocouple chips)
	MAX31855_init();
    
	// Initialize UART (connected to UART-->RS232 chip)
	RS232_Tx_init();			// Tx function

	_EINT();					// Enable general interrupts

}

int __low_level_init(void)
{
  WDTCTL = WDTPW + WDTHOLD;		// Stop WDT
  return (1);					// Force initialization of RAM
}

void update_blinky_led(void)
{
	// lets us know program is running by toggling blinky LED
	P1OUT ^= LED1_PIN;			// Port 1, Pin 0 LED 'blinky LED'
}

void task_25x(void) // 250ms task based on 10ms tick
{
	update_inputs();					// get temperature readings
  	update_uart_250ms();				// broadcast packed message to COM port for use by oven display program
}
void task_50x(void) // 500ms task based on 10ms tick
{
	update_blinky_led();
}

//====================================================================
void main (void)
{
	__low_level_init();					// disable WDT for this appnote

	// Init clocks
	// Init DCO used for clocking everything
	init_dco();

	// Init Digital Hardware
	init_system();

	// Initialize system clock (milliseconds)
	Init_TimerA_SysClock(10);			// base clock initialized to 10 msec
	
	_EINT();							// Enable general interrupts  
  
	uint16 tick_cntr25x   = 0;
	uint16 tick_cntr50x   = 0;

	while (1)
	{
		while(clk_tick == 0){}			// wait until next control loop
		//	while((TACCTL0 & CCIFG) == 0){}
		
		// Add tasks as needed... only need 250 and 500 ms tasks for this demo
		if(0 == tick_cntr25x){task_25x();}
		if(0 == tick_cntr50x){task_50x();}
	
		if(tick_cntr25x < (25-1)){tick_cntr25x++;}		// 250 ms task
		else{tick_cntr25x = 0;}
		if(tick_cntr50x < (50-1)){tick_cntr50x++;}		// 500 ms task
		else{tick_cntr50x = 0;}
		clk_tick = 0;		// clear tick


		// Could wait in low-power mode and return after interrupt in place of clock tick
  } // End of while (1)
} // End of main()
