/***************************************************************************
* File Name: hal430_TmpSnsr.c
* Processor(s): MSP430F16x (tested), MSP430F14x (not tested)
* Compiler: Code Composer Studio V4 (4.2.5.00005)
* 
* Copyright  2012 Playing With Fusion, Inc.
* SOFTWARE LICENSE AGREEMENT: This code is released under the MIT License. 
* 
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
* DEALINGS IN THE SOFTWARE.
* **************************************************************************
* REVISION HISTORY:
* Author			Date		Comments
* J. Steinlage		2012Dec16	Original version
* 
* **************************************************************************
* ADDITIONAL NOTES:
* This file contains functions to configure, read and organize data from
* the MAX31855K-4CH board. 
***************************************************************************/

#include "msp430x16x.h"
#include "msp430_typeDef.h"
#include "pwf_an0001_defines.h"
#include "hal430_TmpSnsr.h"
#include "hal430_spi.h"
#include "pwf_an0001_struct.h"

// thermocouple channels
extern struct var_signed16 temp_TC0;
extern struct var_signed16 temp_TC1;
extern struct var_signed16 temp_TC2;
extern struct var_signed16 temp_TC3;

void MAX31855_init(void)
{
	HalSpiInit_an0001(); 		// initialize MAX31855-specific SPI HAL
}

void update_thermocouples(void)	// configured for use with MAX31855K-4CH board
{
	struct var_signed16 *struct_ptr;	// pointer to a structure

	struct_ptr = &temp_TC0;				// set struct_ptr for thermocouple 0 structure
	read_MAX31855(0, struct_ptr);		// update thermocouple 0
	struct_ptr = &temp_TC1;				// set struct_ptr for thermocouple 1 structure
	read_MAX31855(1, struct_ptr);		// update thermocouple 1
	struct_ptr = &temp_TC2;				// set struct_ptr for thermocouple 2 structure
	read_MAX31855(2, struct_ptr);		// update thermocouple 2
	struct_ptr = &temp_TC3;				// set struct_ptr for thermocouple 3 structure
	read_MAX31855(3, struct_ptr);		// update thermocouple 3
	
}

void read_MAX31855(uint8 thermocoupleID, struct var_signed16 *TC_ptr)
{
	// note: the MAX31855 provides an output of -270 to +1800C with 0.25 deg/bit resolution
	// Accuracy is +/- 2C for temps from -200C to 700C for K-type thermocouples
	// 14 bits are read from the chip, and all are kept.  Scaling must be noted in further processing
	// 4-bytes of output:
	//  D31->D18    D17  D16       D15->D04		 D03     D02          D01         D00
	// thermocuple  RES Fault   RefJunctionTemp  RES  ShortToVCC  ShortToGnd  OpenCircuit
	unsigned int receive1, receive2, receive3, receive4;
	signed int temp;

	//Start by asserting one of the chip select pins 'low'
	if(0 == thermocoupleID){P2OUT &= ~(CS_TC0);}
	else if(1 == thermocoupleID){P2OUT &= ~(CS_TC1);}	
	else if(2 == thermocoupleID){P2OUT &= ~(CS_TC2);}
	else if(3 == thermocoupleID){P2OUT &= ~(CS_TC3);}

	//First Byte from chip
	U0TXBUF = 0x00;					// Send clocks to the chip, this shifts 
									// in the first 8 bits of data.
	complete();						// Wait until all 8 bits have been received
	receive1 = U0RXBUF;				// Store this data in the first data variable
	
	//Second Byte from chip
	U0TXBUF = 0x00;					// Send clocks to the chip, this shifts
									// in the second 8 bits of data.
	complete();						// Wait until all 8 bits have been received. 
	receive2 = U0RXBUF;				// Store this data in the lo-byte variable.

	//Third Byte from chip
	U0TXBUF = 0x00;					// Send clocks to the chip, this shifts
									// in the third 8 bits of data.
	complete();						// Wait until all 8 bits have been received. 
	receive3 = U0RXBUF;				// Store this data in the lo-byte variable.
	
	//Fourth Byte from chip
	U0TXBUF = 0x00;					// Send clocks to the chip, this shifts
									// in the fourth 8 bits of data.
	complete();						// Wait until all 8 bits have been received. 
	receive4 = U0RXBUF;				// Store this data in the lo-byte variable.

	P2OUT |= CS_TC0 + CS_TC1 + CS_TC2 + CS_TC3;		// set chip select pins high to end read, start new measurement
	
	// Begin handling of thermocouple temperature data... internal temp will be handled separately
	if((receive2 & 0x01) != 0)		// STV, STG or Open fault present - don't update TC value, handle fault debounce
	{
		if((0 == TC_ptr->status) && (TC_ptr->fault_cnt < 4))	// status not currently faulted && less than 4 consecutive bad reads
		{
			TC_ptr->fault_cnt++;							// increment fault counter, no status change for less than 4 consecutive bad reads
		}
		else
		{
			if(0x01 == (receive4 & 0x01)){TC_ptr->status = 1;}		// status of 1 indicates 'open' input
			else if(0x02 == (receive4 & 0x02)){TC_ptr->status = 2;}	// status of 2 indicates 'short to ground' input
			else if(0x04 == (receive4 & 0x04)){TC_ptr->status = 3;}	// status of 3 indicates 'short to VCC' input
			else{TC_ptr->status = 4;}								// status of 4 indicates 'unknown fault'
		}
	}														// end thermocouple fault handling
	else									// no Fault detected on thermocouple
	{
		temp = ((0x7F & receive1) << 6) + (receive2 >> 2);	// load 7 MSB from rec1 and 6 LSB from rec2 into temp (13bits)
		if(0x80 == (receive1 & 0x80)){temp |= 0x80;}		// add sign bit back into temp (total 14 bits)
		TC_ptr->value = temp;								// final result loaded into TC_ptr.value
		TC_ptr->status = 0;									// status of 0 indicates signal is OK
		TC_ptr->fault_cnt = 0;								// clear fault counter for specific signal
	}
	// Reference Junction information starts here: 0.0625 DegC/bit resolution, 12-bits (11bit + sign)
	temp = ((0x7F & receive3) << 4) + (receive4 >> 4);	// load 7 MSB from rec 1 and 4 LSB from rec4 into temp (11bits)
	if(0x80 == (receive3 & 0x80)){temp |= 0x8000;}			// add sign bit back into temp (total 12 bits, add as bit15)
	TC_ptr->ref_jcn_temp = temp;
	
}
